from pyNN.utility import get_script_args, init_logging, normalized_filename
simulator_name = get_script_args(1)[0]  
exec("from pyNN.%s import *" % simulator_name)
from maesSpinnV12 import *
from pyNN.utility.plotting import Figure, Panel
import pylab as plt
import numpy as np
output_filename="maes_output.dat"	

intervalAction = 100
setup(timestep=DELAY,min_delay=DELAY,max_delay=DELAY+99,debug=True)

fig_settings = {#'text.usetex':True,
	    'lines.linewidth': 0.5,
	    'axes.linewidth': 0.5,
	    'axes.labelsize': 'small',
	    'legend.fontsize': 'small',
	    'font.size': 8}

plt.rcParams.update(fig_settings)
plt.figure(1, figsize=(8,4))
array_ylabels,array_loc=plt.yticks()
array_loc=(4)
#1.GOALS; 2.GOALS_NP; 3.MODULES; 4. ACTIONS; & 5. FACTS.
def plot_spiketrains(segment, popName):
	sgi=maes1.getMAX_GOALS()
	#sgnpi = maes1.getMAX_GOALS_NP()
	smi = maes1.getMAX_MODULES()
	sai = maes1.getMAX_ACTIONS()
	sfi = maes1.getMAX_FACTS()
	min_y=min([ s.annotations['source_id'] for s in segment.spiketrains])
	max_y=max([ s.annotations['source_id'] for s in segment.spiketrains])
	for spiketrain in segment.spiketrains:		
		if spiketrain is not None:
			y = np.ones_like(spiketrain) * spiketrain.annotations['source_id']
			if (len(y)!=0):
				#print 'minimum y: %d, maximum y: %d' %( min_y,max_y)
#				plt.plot(spiketrain, y, 'r.')
				plt.ylabel(popName)
				if(popName=='Goals'):
					plt.plot(spiketrain, y, 'k.')
					plt.yticks(plt.arange(min_y,max_y,SUB_POPULATION_SIZE))
					plt.yticks(plt.arange(min_y,max_y,SUB_POPULATION_SIZE),maes1.getGoalsList())
				elif(popName=='Modules'):
					plt.plot(spiketrain, y, 'k.')
					plt.yticks(plt.arange(min_y,max_y,SUB_POPULATION_SIZE))
					plt.yticks(plt.arange(min_y,max_y,SUB_POPULATION_SIZE),maes1.getModulesList())
				elif(popName=='Actions'):
					plt.plot(spiketrain, y, 'r.')
					plt.yticks(plt.arange(min_y,max_y,1))
					plt.yticks(plt.arange(min_y,max_y,1),maes1.getActionsList())
				elif(popName=='Facts'):
					plt.plot(spiketrain, y, 'b.')
					plt.yticks(plt.arange(min_y,max_y,SUB_POPULATION_SIZE))
					plt.yticks(plt.arange(min_y,max_y,SUB_POPULATION_SIZE),maes1.getFactsList())
				else:
					plt.setp(plt.gca().get_yticklabels(), visible=True)
		
				#plt.yticks(plt.arange(0,60,10))
				plt.xlabel("time (m/s)" )
				plt.xticks(plt.arange(0,SIM_LENGTH, intervalAction))
				plt.setp(plt.gca().get_xticklabels(), visible=True)
				#plt.legend()
			#else:
			#	print 'length is zero; spiketrain empty'
		else:
			print 'spike train is empty'

def print_spikes(of, label, pop):
	print "\n\n%s\n" % label
	of.write("\n%s\n" % label)
	segment = pop.get_data().segments[0]
	print 'segment'
	print [ s.annotations['source_id'] for s in segment.spiketrains]
	for st in segment.spiketrains:
		y=np.ones_like(st) * st.annotations['source_id']
		for i in range(0,len(st),1):
			#print ('y:%s' %st.annotations['source_id'])
			print ('x:%s,y:%s' %(st[i],st.annotations['source_id']))
			#spike=st[i]+','+st.annotations['source_id']
			of.write('%s,%s\n' %(st.annotations['source_id'],st[i]))

maes1 = Maes() 										# creates Maes net
maes1.networkFromCSV("maes_robot_network1.dat")		# creates network/connections for Maes net
maes1.injectionFromCSV("maes_robot_command.dat")	# creates commands to Maes net
	
#open file
of = open(output_filename, 'w')			# sends output to 'output_filename'
maes1.getActions().recordSpikes() 		# records spikes for Actions
maes1.getGoals().recordSpikes()			# records spikes for Goals
maes1.getModules().recordSpikes()		# records spikes for Modules, optional
maes1.getFacts().recordSpikes()			# records spikes for Facts

#run(SIM_LENGTH, callbacks=[report_time])
run(SIM_LENGTH)
print_spikes(of, "goals", maes1.getGoals().getPop())
print_spikes(of, "modules", maes1.getModules().getPop())
print_spikes(of, "actions", maes1.getActions().getPop())
n_panels = 4
plt.subplot(n_panels, 1, 1)
plot_spiketrains(maes1.getActions().getPop().get_data().segments[0], "Actions")
plt.subplot(n_panels,  1, 2)
plot_spiketrains(maes1.getGoals().getPop().get_data().segments[0], "Goals")
plt.subplot(n_panels, 1, 3)
plot_spiketrains(maes1.getModules().getPop().get_data().segments[0], "Modules")
plt.subplot(n_panels, 1,4)
#plt.subplot(n_panels, 1,3)
plot_spiketrains(maes1.getFacts().getPop().get_data().segments[0], "Facts")
plt.savefig('maesnet_plot')
plt.show()
