#from pyNN import *
#from pyNN.models import *
from pyNN.nest import *
#from pyNN.spiNNaker import *
from nealParams import *
SUB_POPULATION_SIZE	=	5			#size of sub population
	
	
class Oscillator:
	cell_params = {'tau_refrac' : 5.0, 'v_rest' : -65.0,
                    'v_thresh' : -51.0,  'tau_syn_E'  : 2.0, 
                    'tau_syn_I': 5.0,    'v_reset'    : -70.0, 
                    'i_offset' : 0.0, 'cm': 0.1}

	int_weight = 2.500 		#origCRH 2.0
	#int_weight=8.0			#internal weights between neurons in same cell assembly
	inh_weight=1.500		#afferent weights from other cell assemblies - inhibitory
	exc_weight=2.50 		#origCRH 2.0
	#exc_weight=8.00		#efferent weights to other cell assemblies - excitatory
	#constructor
	def __init__(self, label, maxSubPop, int_coeff, inh_coeff, exc_coeff):
		self.setName(label)
		self.int_weight*=int_coeff	
		self.inh_weight*=inh_coeff
		self.exc_weight*=exc_coeff
		if(label=='ACTIONS'):
			self.size= 1 * maxSubPop #actions have only population of size1
			self.subpop=maxSubPop
			self.setPop()
			#no internal connections for actions
		else:
			self.size = SUB_POPULATION_SIZE*maxSubPop
			self.subpop = maxSubPop
			self.setPop()
			self.sub_pop_connect(self.getPop(),self.getPop(), self.int_weight,'excitatory')

	#get
	def getSize(self):
		return self.size
	def getPop(self):
		return self.pop
	def getSubPop(self):
		return self.subpop
	def getName(self):
		return self.name
	def getIntWeight(self):
		return self.int_weight
	def getInhWeight(self):
		return self.inh_weight
	def getExcWeight(self):
		return self.exc_weight
	#set weight
	def setSize(self,x):
		self.size=x
	def setName(self,x):
		self.name=x
	def setSubPop(self,x):
		self.subpop = x
	def setIntWeight(self, w):
		self.int_weight=w
	def setInhWeight(self, w):
		self.inh_weight = w
	def setExcWeight(self, w):
		self.exc_weight=w
	def setName(self, n):
		self.name = n
	#print
	def printOsc(self):
#		print parameters
		print "\n name in Oscillator %s"%self.getName()
		print "\n size in Oscillator %d"%self.getSize()
		print "\n max sub pop  in Oscillator: %d"%self.getSubPop()
	
	#set connections
	def sub_pop_connect(self, fromCells,toCells,weight,inhOrExcite):
		connectors = []
		#spi - sub-population index
		#LBspi - Lower Bound spi
		
		for spi in range(0,self.getSubPop()):	
			LBspi = spi * SUB_POPULATION_SIZE
			for inpNeuron in range (LBspi, LBspi+SUB_POPULATION_SIZE):
				for toNeuron in range ( LBspi,LBspi+SUB_POPULATION_SIZE):
					if (inpNeuron!=toNeuron):
						connectors = connectors+[(inpNeuron,toNeuron,weight,DELAY)]
		excConnector = FromListConnector(connectors)
		Projection(fromCells, toCells, excConnector)
		#SpiNNaker
		#Projection(fromCells, toCells, excConnector, target=inhOrExcite)
	
	#set population - for spinnaker this will require different input source
	def setPop(self):
		self.pop = Population(self.getSize(),IF_curr_exp,self.cell_params)
	def recordSpikes(self):
		return self.getPop().record('spikes')
	def injection(self,start_time,t):
		t*=SUB_POPULATION_SIZE
		#if simulator_name=="spinnaker":
		spikeTimes = [[i for i in range(start_time+1, start_time+PULSE_DURATION,2)]]	
		inputSource = Population(1, SpikeSourceArray, {'spike_times': spikeTimes})
		cons = []
		for k in range(0,5):
			toNeuron = k + t
			cons=cons+[(0,toNeuron,3.5,DELAY)]
		conList=FromListConnector(cons)
		#if simulator_name='spinnaker':
		inhExc='excitatory'
		#Projection(inputSource, self.getPop(), conList,target=inhExc)
		#else:
		Projection(inputSource, self.getPop(), conList)
		#else: 
class Maes:
	goalsList=list()
	actionsList=list()
	goalsNPList=list()
	modulesList=list()
	factsList=list()
	MAX_FACTS,MAX_GOALS,MAX_GOALS_NP,MAX_ACTIONS,MAX_MODULES=0,0,0,0,0
	def __init__(self):
		injectionInterval=100
		currentTime=0
		self.initialiseMaesNetwork()
		#Create the five populations internal weight, inhibitory weight, excitatory weight
		self.Facts = Oscillator('FACTS', self.MAX_FACTS,0.775, 1.0, 1.0)
		self.Goals = Oscillator("GOALS", self.MAX_GOALS,1.0,1.0,1.1)
		self.GoalsNP = Oscillator("GOALSnp", self.MAX_GOALS_NP,1.0,1.0,1.0)
		self.Actions = Oscillator("ACTIONS", self.MAX_ACTIONS,1.0,1.0,1.0)
		self.Modules = Oscillator("MODULES", self.MAX_MODULES,0.9,1.0,1.0)
				

	#get 
	def getMAX_FACTS(self):
		return self.MAX_FACTS
	def getMAX_ACTIONS(self):
		return self.MAX_ACTIONS
	def getMAX_MODULES(self):
		return self.MAX_MODULES
	def getMAX_GOALS(self):
		return self.MAX_GOALS
	def getMAX_GOALS_NP(self):
		return self.MAX_GOALS_NP
	def getNames(self):
		return self.Name
	def getModules(self):
		return self.Modules
	def getActions(self):
		return self.Actions
	def getGoals(self):
		return self.Goals
	def getGoalsNP(self):
		return self.GoalsNP
	def getFacts(self):
		return self.Facts
	def getFactsList(self):
		return self.factsList
	def getGoalsList(self):
		return self.goalsList
	def getActionsList(self):
		return self.actionsList
	def getModulesList(self):
		return self.modulesList
	def getGoalsNPList(self):
		return self.goalsNPList
	def getSubPopIndex(self,spi):  #SPI = sub population index
		return spi*POPULATION_SIZE 
	def getModuleIndex(self, x):
		return self.modulesList.index(x)
	def getActionIndex(self, x):
		return self.actionsList.index(x)
	def getFactIndex(self,x):
		return self.factsList.index(x)
	def getGoalIndex(self,x):
		return self.goalsList.index(x)
	def getGoalNPIndex(self,x):
		return self.goalsNPList.index(x)
	#set up connections
	def connectionSubPop(self, pop1, pop2, sp1, sp2, coeff):
		#print "\n connections: fromType: %s. toType: %s. coeff:%f"%(pop1.getName(), pop2.getName(),coeff)		
		if (pop1.getName()=='ACTIONS'):
			self.connectionSubPopActionOut(pop1,pop2,sp1,sp2,coeff)
		elif (pop2.getName()=='ACTIONS'):
			self.connectionSubPopActionIn(pop1,pop2,sp1,sp2,coeff)
		else:
			self.connectionSubPopNoAction(pop1,pop2,sp1,sp2,coeff)
		
	def connectionSubPopActionOut(self, pop1, pop2, sp1, sp2, coeff):
		inhOrExcite='excitatory'
		cons=[]
		w = coeff*pop1.getExcWeight()
		LBsp2 = sp2*SUB_POPULATION_SIZE #Lower Bound Index for sub population 2
		if (coeff<0):
			inhOrExcite='inhibitory'
			w = coeff*pop1.getInhWeight()
		#AllToAll Connection
		for l in range(LBsp2, LBsp2+SUB_POPULATION_SIZE):
			cons=cons+[(sp1,l,w,DELAY)]
		conList=FromListConnector(cons)
		#nest
		Projection(pop1.getPop(), pop2.getPop(), conList)
		#spiNNaker
		#Projection(pop1.getPop(), pop2.getPop(), conList, target=inhOrExcite)

	def connectionSubPopActionIn(self, pop1, pop2, sp1, sp2, coeff):
		inhOrExcite='excitatory'
		cons=[]
		w = coeff*pop1.getExcWeight()
		LBsp1 = sp1*SUB_POPULATION_SIZE #Lower Bound Index for sub population 2
		if (coeff<0):
			inhOrExcite='inhibitory'
			w = coeff*pop1.getInhWeight()
		#AllToAll Connection
		for l in range(LBsp1, LBsp1+SUB_POPULATION_SIZE):
			cons=cons+[(l,sp2, w,DELAY)]
		conList=FromListConnector(cons)
		#nest
		Projection(pop1.getPop(), pop2.getPop(), conList)
		#spiNNaker
		#Projection(pop1.getPop(), pop2.getPop(), conList, target=inhOrExcite)



	def connectionSubPopNoAction(self, pop1, pop2, sp1, sp2, coeff):
		inhOrExcite='excitatory'
		cons=[]
		w = coeff*pop1.getExcWeight()
		LBsp1 = sp1*SUB_POPULATION_SIZE #Lower Bound Index for sub population 1
		LBsp2 = sp2*SUB_POPULATION_SIZE #Lower Bound Index for sub population 2
		if (coeff<0):
			inhOrExcite='inhibitory'
			w = coeff*pop1.getInhWeight()
		#AllToAll Connection
		for k in range(LBsp1, LBsp1+SUB_POPULATION_SIZE):
			for l in range(LBsp2, LBsp2+SUB_POPULATION_SIZE):
				cons=cons+[(k,l,w,DELAY)]
		conList=FromListConnector(cons)
		#nest
		Projection(pop1.getPop(), pop2.getPop(), conList)
		#spiNNaker
		#Projection(pop1.getPop(), pop2.getPop(), conList, target=inhOrExcite)

	def injectionFromCSV(self, filename):
		import numpy as np
		data = np.genfromtxt(filename,dtype=None, delimiter=',', names=True)
		for item in data:
			print item
			if (item[2].strip()=='FACT'):
				self.Facts.injection(item[0], self.getFactIndex(item[1].strip()))
				
			else:
				self.Goals.injection(item[0], self.getGoalIndex(item[1].strip()))	
    
	def type(self, GMorA):
		if (GMorA.strip()=='FACT'):
			return self.Facts
		elif(GMorA.strip()=='GOAL'):
			return self.Goals
		elif(GMorA.strip()=='MODULE'):
			return self.Modules
		elif(GMorA.strip()=='GOAL_NP'):
			return self.GoalsNP
		elif(GMorA.strip()=='ACTION'):
			return self.Actions
		else:
			print"\n ERROR: type - l242 - %s"%GMorA

	def index(self, GMorA, subpopID):
		if (GMorA.strip()=='FACT'):
			return self.getFactIndex(subpopID)
		elif(GMorA.strip()=='GOAL'):
			return self.getGoalIndex(subpopID)
		elif(GMorA.strip()=='MODULE'):
			return self.getModuleIndex(subpopID)
		elif(GMorA.strip()=='GOAL_NP'):
			return self.getGoalNPIndex(subpopID)
		elif(GMorA.strip()=='ACTION'):
			return self.getActionIndex(subpopID)
		else:
			print"\n ERROR: INDEX - l252 %s"%GMorA

	def networkFromCSV(self, filename):
		import numpy as np
		data = np.genfromtxt(filename, dtype=None,delimiter=',',names=True)
		k=0
		for item in data:
	#		if (item['fromType'].strip()!='#COM') #COMMENTS  are ignored, but may need this in case SpiNNaker is different
			print "%d From  %s.%s to %s.%s @ %f" % (k, item['fromType'],item['subpop1ID'], item['toType'],item['subpop2ID'],item['coeff'])
			self.connectionSubPop(self.type(item['fromType']),self.type(item['toType']), self.index(item['fromType'],item['subpop1ID']) , self.index(item['toType'],item['subpop2ID']),item['coeff'])				
			k+=1		

	def initialiseMaesNetwork(self):
		input_filename= 'maes_robot_instruction_set.dat'
		ip = open(input_filename, 'r')
		allLines=list()
		allLines=ip.read().splitlines()
		print allLines
		i=0
		for line in allLines:
			if(i==0):
				self.goalsList=line.split(',')
				self.MAX_GOALS=len(self.goalsList)		#maximum number of Goals
			elif(i==1):
				self.goalsNPList=line.split(',')
				self.MAX_GOALS_NP=len(self.goalsNPList)		#maximum number of non-primed Goals, for compound statements
			elif(i==2):
				self.modulesList=line.split(',')
				self.MAX_MODULES=len(self.modulesList)		#maximum number of modules, including errors
			elif(i==3):
				self.actionsList=line.split(',')
				self.MAX_ACTIONS=len(self.actionsList)		#maximum number of actions, including errors
			elif(i==4):
				self.factsList=line.split(',')
				self.MAX_FACTS=len(self.factsList)		#maximum number of facts
			else:
				print 'Error: input file'
			i+=1
		ip.close()

